import {FC, useCallback, useEffect, useState} from "react";
import {Form, Image, Input, Picker, Textarea, View} from "@tarojs/components";
import Taro, {getSetting, authorize} from "@tarojs/taro";
import {curriculum, Meeting, meetingAPi} from "@/api";
import {formatDate} from "@/utils/time";
import PopPut from "@/components/popPut/popPut";
import MyButton from "@/components/button/MyButton";
import styles from './spotMeeting.module.scss'
import code from '@/static/img/code.png'
import Icon from "@/components/icon";

const SpotMeeting: FC = () => {
  const path = encodeURIComponent("/pages/meeting/meeting")
  const params = Taro.getCurrentInstance().router?.params as { id: string | undefined }
  const [manages, setManages] = useState<Manage[]>([])
  const [start, setStart] = useState<string>(formatDate(new Date(), "YY-MM-dd hh:00:00"))
  const [end, setEnd] = useState<string>(formatDate(new Date(), "YY-MM-dd 18:00:00"))
  const [depid, setDepid] = useState<number | null>(null)
  const [imgUrl, setImgUrl] = useState('')
  const [name, setName] = useState('')
  const [description, setDescription] = useState('')
  const [loading, setLoading] = useState(false)

  useEffect(() => {
    curriculum.department().then(res => {
      setManages(res.data)
    })

    setLoading(true)
    if (params.id != undefined) {
      meetingAPi.setMeeting(params.id).then(res => {
        if (!Array.isArray(res)) {
          setDepid(res.department_id)
          setName(res.name)
          setDescription(res.description)
          setEnd(formatDate(new Date(res.estimate_end_time), "YY-MM-dd 18:00:00"))
          setStart(formatDate(new Date(res.estimate_start_time), "YY-MM-dd 08:00:00"))
          downUrl(res.id, false)
        } else {
          setLoading(false)
        }
      })
    } else {
      meetingAPi.exists().then(res => {
        if (!Array.isArray(res)) {
          setDepid(res.department_id)
          setName(res.name)
          setDescription(res.description)
          setEnd(formatDate(new Date(res.estimate_end_time), "YY-MM-dd 18:00:00"))
          setStart(formatDate(new Date(res.estimate_start_time), "YY-MM-dd 08:00:00"))
          downUrl(res.id, false)
        } else {
          setLoading(false)
        }
      })
    }
  }, [])

  const change = useCallback((e) => {
    setDepid(() => manages[Number(e.detail.value)]?.id)
  }, [manages])

  const change_start = useCallback((time: string) => {
    const mil = new Date(time).getTime()
    const endMil = new Date(end).getTime()
    if (mil > endMil) {
      Taro.showToast({title: '大于结束时间', icon: 'error'})
    } else {
      setStart(time)
    }
  }, [end])

  const change_end = useCallback((time: string) => {
    const mil = new Date(time).getTime()
    const startMil = new Date(start).getTime()
    if (mil < startMil) {
      Taro.showToast({title: '小于开始时间', icon: 'error'})
    } else {
      setEnd(time)
    }
  }, [start])

  const handleWriteFile = useCallback(() => {
    if (imgUrl == null) {
      Taro.showToast({title: '下载失败', icon: 'error'})
      return
    }
    Taro.saveImageToPhotosAlbum({
      filePath: imgUrl,
      success() {
        Taro.showModal({title: '下载成功'})
      },
      fail() {
        Taro.showToast({title: '下载失败', icon: 'error'})
      }
    })
  }, [imgUrl])

  const handleSaveCode = useCallback(() => {
    if (process.env.TARO_ENV === 'h5') {
      Taro.showToast({title: '长按保存二维码', icon: 'error'})
    } else {
      getSetting({
        success: function ({authSetting}) {
          //没有权限则申请
          if (!authSetting["scope.writePhotosAlbum"]) {
            authorize({
              scope: "scope.writePhotosAlbum",
              success: () => {
                //存储二维码
                handleWriteFile()
              },
            });
          } else {
            handleWriteFile()
          }
        },
      });
    }
  }, [imgUrl]);

  const downUrl = (id: number, down = true) => {
    Taro.showLoading({title: '加载二维码'})
    const url = process.env.TARO_APP_API + '/official/qrcode?' + 'meeting_id=' + id + '&path=' + path

    if (process.env.TARO_ENV !== 'h5') {
      Taro.downloadFile({
        url,
        success(res) {
          setImgUrl(res.tempFilePath)
          if (down) {
            handleSaveCode()
          }
        },
        fail() {
          Taro.showModal({title: '二维码生成失败'})
        },
        complete() {
          setLoading(false)
          Taro.hideLoading()
        }
      })
    } else {
      setTimeout(() => {setImgUrl(url)})
      if (down) {handleSaveCode()}
      Taro.hideLoading()
      setLoading(false)
    }
  }


  async function submit(e) {
    if (imgUrl) { // 已有
      handleSaveCode()
      return
    }

    const values: Meeting = e.detail.value
    for (const [key, value] of Object.entries(values)) {
      if (!value && key !== 'description') {
        Taro.showToast({title: '请认真填写', icon: 'error'})
        return
      }
    }
    if (!depid) {
      Taro.showToast({title: '请选择部门', icon: 'error'})
      return
    }

    setLoading(true)
    try {
      const res = await meetingAPi.setMeetings({
        ...values,
        estimate_end_time: new Date(end).getTime(),
        estimate_start_time: new Date(start).getTime(),
        department_id: depid
      })
      downUrl(res.id)
    } catch (e) {

    }
    setLoading(false)
  }

  function jumpMeeings() {
    Taro.navigateTo({url: '/pages/manage/meetings/meetings'})
  }

  function imageOnLoad() {
    Taro.hideLoading()
  }

  useEffect(() => {
    if (imgUrl) {
      setImgUrl('')
    }
  }, [start, end, name, depid, description])

  return (
    <View className={styles.page}>

      <View className={styles.box}>
        <View>填完信息才能生成二维码</View>

        <View className={styles.code}>
          {
            imgUrl
              ? <Image src={imgUrl} mode='aspectFit' onLoad={imageOnLoad}/>
              : <Image src={code} mode='aspectFit'/>
          }
        </View>

        <View className={styles.choice}>
          <Form onSubmit={submit}>
            <PopPut
              title='会议标题'
              content={<Input
                className='input'
                name='name'
                value={name}
                disabled={!!params.id}
                placeholder='请输入会议标题'
                onInput={(e) => setName(e.detail.value)}
              />}
              chevron
            />
            <Picker
              mode="date"
              value={start}
              disabled={!!params.id}
              onChange={(e) => change_start(e.detail.value + ' 8:00:00')}
              name='estimate_start_time'>
              <PopPut title='开始时间' content={start}/>
            </Picker>
            <Picker
              mode="date"
              value={end}
              disabled={!!params.id}
              onChange={(e) => change_end(e.detail.value + ' 18:00:00')}
              name='estimate_end_time'>
              <PopPut title='结束时间' content={end}/>
            </Picker>
            <Picker
              mode='selector'
              range={manages}
              onChange={change}
              disabled={!!params.id}
              rangeKey='name'
              name='department_id'>
              <PopPut title='选择部门' content={manages?.find(x => x.id == depid)?.name}/>
            </Picker>

            <Textarea
              value={description}
              className='Textarea'
              disabled={!!params.id}
              placeholder='请输入描述'
              maxlength={255}
              name='description'
              onInput={(e) => setDescription(e.detail.value)}
              style={{height: '60px'}}>
            </Textarea>


            <MyButton
              formType='submit'
              fillet
              width={200}
              className={styles.button}
              loading={loading}>
              保存二维码
            </MyButton>

          </Form>
        </View>
        <View className='text-center text-muted my-3' onClick={jumpMeeings}>历史列表 <Icon name='chevron-right'/></View>
      </View>

    </View>
  )
}

export default SpotMeeting
